/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.crop.model;

import java.awt.Color;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import javax.swing.event.UndoableEditEvent;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import nl.cloudfarming.client.model.AbstractEntity;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.awt.UndoRedo;
import org.openide.util.Exceptions;

/**
 * PropertyChangeSupport for the crop model
 *
 * @author Merijn Zengers
 */
public abstract class CropBean extends AbstractEntity implements Serializable {

    private static final Color DEFAULT_COLOR = Color.LIGHT_GRAY;
    private String name;
    public static final String PROP_NAME = "name";
    private Color color;
    public static final String PROP_COLOR = "color";
    private String source;
    public static final String PROP_SOURCE = "source";
    private transient List<UndoRedo.Manager> undoRedoManagers;
    public transient BeanState currentState = BeanState.NORMAL;

    public CropBean(EnterpriseIdProvider eip, ItemIdType itemIdType) {
        super(eip, itemIdType);
    }

    public CropBean(EnterpriseIdProvider eip, ItemIdType itemIdType, String name) {
        super(eip, itemIdType);
        this.name = name;
    }

    public CropBean(EnterpriseIdProvider eip, ItemIdType itemIdType, String name, String source) {
        super(eip, itemIdType);
        this.source = source;
        this.name = name;
    }

    // preferring copy constructor over cloning to get default initialization of 
    // transient properties like listeners, beanstate etc.
    public CropBean(CropBean other) {
        this(other.getURI(), other.getItemIdType(), other.getName(), other.getSource());
        color = other.getColor();
    }

    /**
     * Get the value of source
     *
     * @return the value of source
     */
    public String getSource() {
        return source;
    }

    /**
     * Set the value of source
     *
     * @param source new value of source
     */
    public void setSource(String source) {
        String oldSource = this.source;
        this.source = source;
        getPropertyChangeSupport().firePropertyChange(PROP_SOURCE, oldSource, source);
    }

    /**
     * Set the value of color
     *
     * @param source new value of source
     */
    public void setColor(Color color) {
        Color oldColor = this.color;
        this.color = color;
        getPropertyChangeSupport().firePropertyChange(PROP_COLOR, oldColor, color);
    }

    public Color getColor() {
        if (color == null) {
            return DEFAULT_COLOR;
        }
        return color;
    }

    public void setName(String name) {
        String old = this.name;
        this.name = name;
        getPropertyChangeSupport().firePropertyChange(PROP_NAME, old, name);
        fireUndoableEvent(PROP_NAME, this, old, name);
    }

    public String getName() {
        return name;
    }

    @Override
    public String toString() {
        return name;
    }

    private void setCurrentState(BeanState currentState) {
        this.currentState = currentState;
    }

    private BeanState getCurrentState() {
        return currentState;
    }

    public enum BeanState {

        NORMAL, UNDO
    }

    private List<UndoRedo.Manager> getUndoRedoManagers() {
        if (undoRedoManagers == null) {
            undoRedoManagers = new ArrayList<>();
        }
        return undoRedoManagers;
    }

    public void addUndoableEditListener(UndoRedo.Manager manager) {
        getUndoRedoManagers().add(manager);
    }

    public void removeUndoableEditListener(UndoRedo.Manager manager) {
        getUndoRedoManagers().remove(manager);
    }

    public void fireUndoableEvent(String property, CropBean source, Object oldValue, Object newValue) {
        if (getCurrentState() != BeanState.UNDO) {
            for (UndoRedo.Manager manager : getUndoRedoManagers()) {
                if (manager != null) {
                    ReUndoableEdit reUndoableEdit = new ReUndoableEdit(property, source, oldValue, newValue);
                    UndoableEditEvent undoableEditEvent = new UndoableEditEvent(this, reUndoableEdit);


                    assert undoableEditEvent != null;

                    manager.undoableEditHappened(undoableEditEvent);
                }
            }
        }
    }

    /*
     * Custom UndoableEdit extending abstractUndoableEdit Custom implemented
     * Undo & Redo functionality Will call setters on bean using reflection when
     * undo or redo is called.
     *
     *
     * @Author Gerben Feenstra & Merijn Zengers
     *
     */
    private class ReUndoableEdit extends AbstractUndoableEdit {

        private Object oldValue;
        private Object newValue;
        private CropBean source;
        private String property;

        public ReUndoableEdit(String property, CropBean source, Object oldValue, Object newValue) {
            super();
            this.oldValue = oldValue;
            this.newValue = newValue;
            this.source = source;
            this.property = property;
        }

        private Method getMethod() {
            try {
                return source.getClass().getMethod("set" + property.replaceFirst(property.substring(0, 1), property.substring(0, 1).toUpperCase()), newValue.getClass());
            } catch (NoSuchMethodException | SecurityException ex) {
                // TODO: FIX
                Exceptions.printStackTrace(ex);
            }
            return null;
        }

        @Override
        public void undo() throws CannotUndoException {
            try {
                super.undo();
                Method method = getMethod();
                setCurrentState(BeanState.UNDO);
                method.invoke(source, oldValue);
                setCurrentState(BeanState.NORMAL);
            } catch (SecurityException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
                // TODO: FIX
                Exceptions.printStackTrace(ex);
            }
        }

        @Override
        public void redo() throws CannotRedoException {
            try {
                super.redo();
                Method method = getMethod();
                setCurrentState(BeanState.UNDO);
                method.invoke(source, newValue);
                setCurrentState(BeanState.NORMAL);
            } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
    }
}
