/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.crop.model;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;

/**
 * A CropDefinition contains the possible Varieties and Classifications for a
 * crop.
 *
 * @author Wytse Visser
 */
public class CropDefinition extends CropBean implements Serializable, PropertyChangeListener {
    
    public static final String EXT = "ctn";
    private Crop crop;
    public static final String PROP_VARIETIES = "varieties";
    private CropBeans<Variety> varieties;
    public static final String PROP_CLASSIFICATIONS = "classifications";
    private CropBeans<Classification> classifications;
    
    public CropDefinition(EnterpriseIdProvider eip, Crop crop) {
        super(eip, ItemIdType.CTN);
        setCrop(crop);
        varieties = new CropBeans(PROP_VARIETIES);
        classifications = new CropBeans(PROP_CLASSIFICATIONS);
    }
    
    public final void setCrop(Crop crop) {
        this.crop = crop;
        crop.addPropertyChangeListener(this);
    }
    
    public Crop getCrop() {
        return crop;
    }
    
    @Override
    public void setColor(Color color) {
        getCrop().setColor(color);
    }
    
    @Override
    public Color getColor() {
        return getCrop().getColor();
    }
    
    public void setClassifications(List<Classification> newClassifications) {
        classifications.setCropBeans(newClassifications);
    }
    
    public List<Classification> getClassifications() {
        return classifications.toList();
    }
    
    public void addClassification(Classification classification) {
        classifications.add(classification);
    }
    
    public void removeClassification(Classification classification) {
        classifications.remove(classification);
    }
    
    public void setVarieties(List<Variety> newVarieties) {
        varieties.setCropBeans(newVarieties);
    }
    
    public List<Variety> getVarieties() {
        return varieties.toList();
    }
    
    public void addVariety(Variety variety) {
        varieties.add(variety);
    }
    
    public void removeVariety(Variety variety) {
        varieties.remove(variety);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        getPropertyChangeSupport().firePropertyChange(evt);
    }
    
    @Override
    public String toString() {
        return getName();
    }
    
    @Override
    public String getName() {
        return getCrop().getName();
    }
    
    @Override
    public void setName(String name) {
        getCrop().setName(name);
    }
    
    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        super.addPropertyChangeListener(listener);
        varieties.addPropertyChangeListener(listener);
        classifications.addPropertyChangeListener(listener);
    }

    /**
     * This class provides functionality for managing a collection of CropBeans.
     * It automatically fires a PropertyChangeEvent if its contents change.
     *
     * @param <T> The type of CropBean. Can be either a Variety or a
     * Classification.
     */
    private class CropBeans<T extends CropBean> implements Serializable {
        
        private final String propertyName;
        private List<T> cropBeans;
        
        public CropBeans(String propertyName) {
            this.propertyName = propertyName;
            cropBeans = new ArrayList<>();
        }
        
        void add(T cropBean) {
            List<T> old = cropBeans;
            cropBeans.add(cropBean);
            getPropertyChangeSupport().firePropertyChange(propertyName, null, cropBean);
            fireUndoableEvent(propertyName, CropDefinition.this, old, cropBeans);
        }
        
        void remove(T cropBean) {
            List<T> old = cropBeans;
            cropBeans.remove(cropBean);
            getPropertyChangeSupport().firePropertyChange(propertyName, cropBean, null);
            fireUndoableEvent(propertyName, CropDefinition.this, old, cropBeans);
        }
        
        void setCropBeans(List<T> cropBeans) {
            List<T> old = new ArrayList<>(cropBeans);
            this.cropBeans = cropBeans;
            getPropertyChangeSupport().firePropertyChange(propertyName, old, cropBeans);
        }
        
        List<T> toList() {
            return cropBeans;
        }
        
        void addPropertyChangeListener(PropertyChangeListener listener) {
            for (CropBean cropBean : cropBeans) {
                cropBean.addPropertyChangeListener(listener);
            }
        }
    }
}
