/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.crop.model;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import nl.cloudfarming.client.model.AgroURI;
import org.openide.util.Lookup;
import org.openide.util.WeakListeners;

/**
 * <P>An crop instance represents a Crop with it's classification and it's
 * variety</p> <p>It holds links to the original crop from which it was created
 * via the uri of the crop</p>
 *
 * @author Wytse Visser
 */
public class CropInstance implements Serializable {

    private transient PropertyChangeSupport propertyChangeSupport;
    public static final String PROP_COLOR = "color";
    private AgroURI cropURI;
//    private AgroURI varietyURI;
//    private AgroURI classificationURI;
    private Variety variety;
    private Classification classification;
    private transient CropColorChangeListener changeListener;

    /**
     * <p>Creates a CropInstance with a crop a variety and a classification</p>
     * The passed crop needs to have a URI set otherwise an {@link IllegalArgumentException}
     * is thrown
     *
     * @param crop
     * @param variety
     * @param classification
     */
    public CropInstance(Crop crop, Variety variety, Classification classification) {
        if (crop.getURI() == null) {
            throw new IllegalArgumentException("Crop needs an URI set");
        }
        setCrop(crop);
        setVariety(variety);
        setClassification(classification);
        initCropColorListener();
    }

    private void initCropColorListener() {
        Crop crop = getCrop();
        if (crop != null) {
            changeListener = new CropColorChangeListener();
            crop.addPropertyChangeListener(Crop.PROP_COLOR, WeakListeners.propertyChange(changeListener, crop));
        }
    }

    public final void setCropDefinition(CropDefinition cropDefinition) {
        setCrop(cropDefinition.getCrop());
    }

    public final void setCrop(Crop crop) {
        this.cropURI = crop.getURI();
    }

    /**
     * Tries to find the crop of this CropInstance via the uri of the crop
     *
     * @return
     */
    public Crop getCrop() {
        if (cropURI != null) {
            CropManager manager = Lookup.getDefault().lookup(CropManager.class);
            return manager.find(cropURI);
        }
        return null;
    }

    public final void setVariety(Variety variety) {
        this.variety = variety; // FIXME: URIfy + add listener -WV
    }

    public Variety getVariety() {
        return variety; // FIXME:URIfy
//        VarietyManager varietyManager = Lookup.getDefault().lookup(VarietyManager.class);
//        return varietyManager.find(varietyURI);
    }

    public final void setClassification(Classification classification) {
        this.classification = classification;// FIXME: URIfy + add listener -WV
    }

    public Classification getClassification() {
        return classification; // FIXME: URIfy 
    }

    public Color getColor() {
        return getCrop().getColor();
    }

    public String getName() {
        return getCrop().getName();
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        initCropColorListener();
    }

    private class CropColorChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            if (Crop.PROP_COLOR.equals(pce.getPropertyName())) {
                getPropertyChangeSupport().firePropertyChange(PROP_COLOR, pce.getOldValue(), pce.getNewValue());
            }
        }
    }
}
